/* A sample program that exercises the msTokenize*() functions.

This has been successfully compiled using the following command:

g++ -g -fPIC -Wall -fprofile-arcs -ftest-coverage -DHAVE_VSNPRINTF \
-DNEED_STRLCPY -DNEED_STRLCAT -DNEED_STRRSTR -DUSE_LIBXML2 \
-DUSE_AGG_SVG_SYMBOLS -DUSE_GD_GIF -DUSE_GD_PNG -DUSE_GD_JPEG \
-DUSE_GD_WBMP -DUSE_GD_FT -DGD_HAS_FTEX_XSHOW -DGD_HAS_GDIMAGEGIFPTR \
-DGD_HAS_GETBITMAPFONTS -DGD_HAS_GET_TRUE_COLOR_PIXEL -DUSE_ICONV \
-DUSE_GIF -DUSE_PNG -DUSE_ZLIB -DUSE_FREETYPE -DDISABLE_CVSID \
-I/usr/include/freetype2 -I/usr/include -Irenderers/agg/include \
-I/usr/include/libxml2 tokenize-test.c -L. -lmapserver -lfreetype \
-lz -lbz2 -lgd -L/usr/lib -lgd -ljpeg -lfreetype -lz -lbz2 -lpng -lz \
-lXpm -lX11 -lexpat -ljpeg -lpng -lgif -lc -lz -lxml2 -lz -lm -lm \
-lstdc++ -o tokenize-test

Running valgrind over this program does not reveal any memory leaks:

valgrind --leak-check=full --show-reachable=yes ./tokenize-test tests/test.map  > /dev/null 2

==14295== Memcheck, a memory error detector
==14295== Copyright (C) 2002-2010, and GNU GPL'd, by Julian Seward et al.
==14295== Using Valgrind-3.6.1 and LibVEX; rerun with -h for copyright info
==14295== Command: ./tokenize-test tests/test.map 2
==14295== 
==14295== 
==14295== HEAP SUMMARY:
==14295==     in use at exit: 0 bytes in 0 blocks
==14295==   total heap usage: 57 allocs, 57 frees, 20,064 bytes allocated
==14295== 
==14295== All heap blocks were freed -- no leaks are possible
==14295== 
==14295== For counts of detected and suppressed errors, rerun with: -v
==14295== ERROR SUMMARY: 0 errors from 0 contexts (suppressed: 51 from 8)

 */

#include "mapserver.h"

const char *mapfile = "MAP\n\
  NAME \"Testing\"\n\
  EXTENT -0.5 50.977222 0.5 51.977222\n\
  #IMAGETYPE JPEG\n\
  #IMAGETYPE PNG24\n\
  IMAGETYPE PNG\n\
  IMAGECOLOR 255 255 255\n\
  STATUS ON\n\
  SIZE 200 200\n\
  FONTSET \"fonts.txt\"\n\
  SYMBOLSET \"symbols.txt\"\n\
\n\
  SCALEBAR\n\
    STATUS EMBED\n\
    POSTLABELCACHE TRUE\n\
    STYLE 0\n\
    UNITS METERS\n\
    SIZE 150 3\n\
    POSITION LR\n\
    TRANSPARENT TRUE\n\
    COLOR 0 0 0\n\
    IMAGECOLOR 242 255 195\n\
    BACKGROUNDCOLOR 255 255 255\n\
    LABEL\n\
      TYPE BITMAP\n\
      SIZE TINY\n\
      COLOR 0 0 0\n\
      POSITION UR\n\
      BUFFER 10\n\
    END\n\
  END\n\
\n\
  LEGEND\n\
    IMAGECOLOR 255 255 255\n\
  END\n\
  \n\
  WEB\n\
    METADATA\n\
      \"key1\" \"value1\"\n\
      \"key2\" \"value2\"\n\
      \"key3\" \"value3\"\n\
      \"key4\" \"value4\"\n\
    END\n\
  END\n\
\n\
  PROJECTION\n\
    \"init=epsg:4326\"\n\
  END\n\
 \n\
  LAYER\n\
    NAME \"RASTER\"\n\
    TYPE RASTER\n\
    PROJECTION\n\
      \"init=epsg:4326\"\n\
    END\n\
    DATA \"raster.tif\"\n\
    PROCESSING \"BANDS=1\"\n\
    STATUS DEFAULT\n\
  END\n\
  \n\
  LAYER\n\
    NAME \"POLYGON\"\n\
    TYPE POLYGON\n\
    PROJECTION\n\
      \"init=epsg:4326\"\n\
    END\n\
    STATUS DEFAULT\n\
    DATA \"polygon\"\n\
    METADATA\n\
      \"key1\" \"value1\"\n\
      \"key2\" \"value2\"\n\
      \"key3\" \"value3\"\n\
      \"key4\" \"value4\"\n\
    END\n\
    CLASSITEM \"FNAME\"\n\
    CLASS\n\
      NAME \"0\"\n\
      METADATA\n\
        \"key1\" \"value1\"\n\
        \"key2\" \"value2\"\n\
        \"key3\" \"value3\"\n\
        \"key4\" \"value4\"\n\
      END\n\
      STYLE\n\
        COLOR 255 153 102\n\
        OUTLINECOLOR 0 0 204\n\
        SYMBOL 1\n\
        SIZE 2\n\
      END\n\
    END\n\
    CLASS\n\
      NAME \"1\"\n\
      EXPRESSION \"foo\"\n\
      STYLE\n\
        COLOR 255 153 102\n\
        OUTLINECOLOR 0 0 204\n\
        SYMBOL 1\n\
        SIZE 2\n\
      END\n\
    END\n\
  END\n\
\n\
  LAYER\n\
    NAME \"LINE\"\n\
    TYPE LINE\n\
    PROJECTION\n\
      \"init=epsg:4326\"\n\
    END\n\
    TRANSPARENCY 1\n\
    STATUS DEFAULT\n\
    DATA \"line\"\n\
    CLASSITEM \"FNAME\"\n\
    CLASS\n\
      NAME \"0\"\n\
      STYLE\n\
        COLOR 0 153 0\n\
        SYMBOL 1\n\
        SIZE 3\n\
      END\n\
    END\n\
    CLASS\n\
      NAME \"1\"\n\
      EXPRESSION \"foo\"\n\
      STYLE\n\
        COLOR 0 153 0\n\
        SYMBOL 1\n\
        SIZE 3\n\
      END\n\
    END\n\
  END\n\
\n\
  LAYER\n\
    NAME \"POINT\"\n\
    EXTENT -0.5 51.0 0.5 52.0\n\
    TYPE POINT\n\
    PROJECTION\n\
      \"init=epsg:4326\"\n\
    END\n\
    STATUS DEFAULT\n\
    DATA \"point\"\n\
    CLASSITEM \"FNAME\"\n\
    LABELITEM \"FNAME\"\n\
    CLASS\n\
      NAME \"0\"\n\
      STYLE\n\
        COLOR 0 0 0\n\
        SYMBOL 1\n\
        SIZE 13\n\
      END\n\
      STYLE\n\
        COLOR 204 204 204\n\
        SYMBOL 1\n\
        SIZE 7\n\
      END\n\
      LABEL\n\
        TYPE TRUETYPE\n\
        FONT \"Vera\"\n\
        SIZE 10\n\
        COLOR 0 0 0\n\
      END\n\
\n\
      TEMPLATE \"point.html\"\n\
    END\n\
    CLASS\n\
      NAME \"1\"\n\
      EXPRESSION \"foo\"\n\
      STYLE\n\
        COLOR 0 0 0\n\
        SYMBOL 1\n\
        SIZE 13\n\
      END\n\
    END\n\
  END\n\
 \n\
  LAYER\n\
    NAME \"INLINE\"\n\
    TYPE POINT\n\
    PROJECTION\n\
      \"init=epsg:4326\"\n\
    END\n\
    STATUS DEFAULT\n\
    FEATURE\n\
      POINTS -0.2 51.5 END\n\
    END\n\
    CLASS\n\
      NAME \"0\"\n\
      STYLE\n\
        COLOR 0 0 0\n\
        SYMBOL 1\n\
        SIZE 3\n\
      END\n\
    END\n\
  END\n\
\n\
  LAYER\n\
    NAME \"INLINE-PIXMAP-RGBA\"\n\
    TYPE POINT\n\
    TRANSPARENCY ALPHA\n\
    DEBUG ON\n\
    PROJECTION\n\
      \"init=epsg:4326\"\n\
    END\n\
    STATUS OFF\n\
    FEATURE\n\
      POINTS 0.2 51.7 END\n\
    END\n\
    CLASS\n\
      NAME \"0\"\n\
      STYLE\n\
        SYMBOL 'xmarks-png'\n\
      END\n\
      LABEL\n\
        COLOR 0 0 0\n\
      END\n\
    END\n\
  END\n\
\n\
LAYER\n\
    NAME \"INLINE-PIXMAP-PCT\"\n\
    TYPE ANNOTATION\n\
    DEBUG ON\n\
    TRANSPARENCY ALPHA\n\
    PROJECTION\n\
      \"init=epsg:4326\"\n\
    END\n\
    STATUS DEFAULT\n\
    FEATURE\n\
      POINTS 0.2 51.2 END\n\
    END\n\
    CLASS\n\
      NAME \"0\"\n\
      STYLE\n\
        SYMBOL 'home-png'\n\
      END\n\
      TEXT \".\"\n\
      LABEL\n\
        COLOR 0 0 0\n\
      END\n\
    END\n\
  END\n\
\n\
END\n\
";

void printTokens(char **tokens, int numtokens) {
  int i;

  for (i=0; i < numtokens; i++) {
    printf("TOKEN: %s\n", tokens[i]);
  }

  msFreeCharArray(tokens, numtokens);
}

int main(int argc, char *argv[])
{
  char **tokens = NULL;
  char *filename = NULL;
  int numtokens;
  FILE *stream = NULL;

  if (argc != 2) {
    printf("Usage: tokenize-test path/to/mapfile.map\n");
    return (1);
  }
  filename = argv[1];

  printf("Output tokens from msTokenizeMap():\n");
  if ((tokens = msTokenizeMap(filename, &numtokens)) == NULL) {
    printf("No tokens returned!\n");
  } else {
    printTokens(tokens, numtokens);
  }

  printf("\nOutput tokens from msTokenizeMapStream():\n");
  if((stream = fopen(filename,"r")) == NULL) {
    printf("Cannot open %s", filename);
  } else if ((tokens = msTokenizeMapStream(stream, &numtokens)) == NULL) {
    printf("No tokens returned!");
    fclose(stream);
  } else {
    printTokens(tokens, numtokens);
    fclose(stream);
  }

  printf("\nOutput tokens from msTokenizeMapString():\n");
  if ((tokens = msTokenizeMapString((char *)mapfile, &numtokens)) == NULL) {
    printf("No tokens returned!\n");
  } else {
    printTokens(tokens, numtokens);
  }

  return (0);
}
